/**
 * \file mgui/hotkeyset.h
 * \brief MGUI::HOTKEYSET definitions.
 *
 * \if NODOC
 * $Id: hotkeyset.h_v 1.4 2011/12/02 22:21:48 mju Exp $
 *
 * $Log: hotkeyset.h_v $
 * Revision 1.4  2011/12/02 22:21:48  mju
 * Remove redundant hdr.
 *
 * Revision 1.3  2008/04/10 22:12:53  scowan
 * Added isEmpty method.
 *
 * Revision 1.1  2007/11/29 22:34:08  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_HOTKEYSET_H
#define	INC_MGUI_HOTKEYSET_H

#ifndef	INC_MGUI_KEYCODE_H
	#include <mgui/keycode.h>
#endif

#ifndef	INC_MGUI_KEYSTATE_H
	#include <mgui/keystate.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

namespace MGUI {

class HOTKEYSET {
	public:
		typedef FastDelegate<void(KEYCODE, KEYSTATE, const POINT&)> DELEGATE;
	
		//! Default ctor
		HOTKEYSET ();

		//! Default copy ctor
		HOTKEYSET (
			const HOTKEYSET& rhs
			);

		//! Destructor
		~HOTKEYSET ();
			
		//! Assignment operator
		HOTKEYSET& operator= (
			const HOTKEYSET& rhs
			);

		//! Add a hot key and a delegate to call to this set of hot keys
		//! @return 'True' if hot key entry added, 'false' if one already exists
		bool Add (
			KEYCODE KeyCode,
			DELEGATE HotKeyDelegate,
			KEYSTATE KeyState = KEYSTATE_None
			);

		//! Add another HOTKEYSET to this set.
		//! @return 'True' if all hot key entries added, 'false' if there is one or more keys that match.
		bool Add (
			const HOTKEYSET& HotKeySet
			);

		//! Find the entry in this set based on the KeyCode and KeyStsate and call the delegate associated with it.
		//! @return 'True' if entry is found and delegate called, 'false' if entry not found.
		bool CallDelegate (
			KEYCODE KeyCode,
			KEYSTATE KeyState,
			const POINT& Point			//!< Point where the mouse was located in the MGUI::CTRL or MGUI::SHELL the hot key was pressed
			) const;
			
		//! Determine if this set of hot keys is empty.
		//! @return 'True' if set is empty, 'false' if not.
		bool IsEmpty (
			) const { return (m_HotKeyItemList.IsEmpty()); }

		//! Remove a hot key entry, must match all three parameters.
		//! @return 'True' if hot key entry removed, 'false' if did not exist in set.
		bool Remove (
			KEYCODE KeyCode,
			DELEGATE HotKeyDelegate,
			KEYSTATE KeyState = KEYSTATE_None
			);

		//! Add a set of hot key's from this set.  An entry must match all three parameters in the single Remove() method listed above.
		//! @return 'True' if all hot key entries removed, 'false' if there is one or more keys that does not match
		bool Remove (
			const HOTKEYSET& HotKeySet
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		struct ITEM {
			KEYCODE m_KeyCode;
			KEYSTATE m_KeyState;
			DELEGATE m_HotKeyDelegate;
			};

		MILIST<ITEM> m_HotKeyItemList;
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_HOTKEYSET_H
