/**
 * \file mgui/popupradio.h
 * \brief MGUI::POPUP_RADIO definitions
 *
 * \if NODOC
 * $Id: popupradio.h_v 1.2 2011/11/09 15:11:22 mju Exp $
 *
 * $Log: popupradio.h_v $
 * Revision 1.2  2011/11/09 15:11:22  mju
 * Fix dox warning.
 *
 * Revision 1.1  2011/08/25 21:17:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_POPUPRADIO_H
#define	INC_MGUI_POPUPRADIO_H

#ifndef	INC_MGUI_POPUP_H
#include <mgui/popup.h>
#endif

#ifndef	INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif

namespace MGUI {

//=====================================================================================================================
//! Popup showing set of mutually-exclusive (radio) icon buttons.
class POPUP_RADIO : public POPUP {
	public:

		//! Constructor.
		explicit POPUP_RADIO (
			LAYOUT_ORIENTATION orientation = LAYOUT_ORIENTATION_Horizontal,
			int NumPerRowOrColumn = 0,					//!< Number of buttons per row or column if multiple rows/columns
			CTRL_TOGGLEBUTTON::STYLE ToggleStyle = CTRL_TOGGLEBUTTON::STYLE_Radio
			);

		//! Destructor.
		virtual ~POPUP_RADIO ();

		//! Add button with ICON.
		//! Must be done before popup is first created or opened.
		void AddButton (
			ID id,							//!< ID to associate with button
			const ICON& icon,				//!< Icon for button
			const MISTRING& tooltip		//!< ToolTip
			);

		//! Get ICON for current selection.
		const ICON& GetSelectedIcon () const
			{ return (m_Form.GetSelectedIcon()); }

		//! Get ID of current selection.
		ID GetSelectedID () const
			{ return (m_Form.GetSelectedID()); }

		//! Set current selection.
		//! If not button has specified ID then no button will appear as selected.
		void SetSelectedID (
			ID id								//!< ID used in AddButton
			) { m_Form.SetSelectedID(id); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef POPUP_RADIO THISCLASS;

		FORM_RADIOBUTTONS m_Form;
		LAYOUT_ORIENTATION m_orientation;
		int m_NumPerRowOrColumn;

		void OnSelection ();

		virtual ERRVALUE v_CreateContent ();
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_POPUPRADIO_H
