/**
 * \file mi32/bool3.h
 * \brief BOOL3 definitions
 *
 * \if NODOC
 * $Id: bool3.h_v 1.4 2011/07/21 13:37:50 mju Exp $
 *
 * $Log: bool3.h_v $
 * Revision 1.4  2011/07/21 13:37:50  mju
 * Rearrange to attempt to fix doxygen parsing.
 *
 * Revision 1.3  2011/07/20 19:42:24  mju
 * Fix op= to return value.
 *
 * Revision 1.2  2011/07/20 19:32:35  mju
 * Add ctor/op= from bool.
 *
 * Revision 1.1  2011/07/20 18:00:45  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_BOOL3_H
#define	INC_MI32_BOOL3_H


//! BOOL3 enumeration values.
enum BOOL3_VALUE {
	BOOL3_Unknown = -1,		//!< Unknown, undefined, etc.
	BOOL3_False = 0,			//!< False, no, etc.
	BOOL3_True = 1				//!< True, yes, etc
	};

//! Logical negation.
inline BOOL3_VALUE operator! (BOOL3_VALUE rhs)
	{ return ((rhs == BOOL3_False) ? BOOL3_True : (rhs == BOOL3_True) ? BOOL3_False : BOOL3_Unknown); }

//! Logical and.
inline BOOL3_VALUE operator&& (BOOL3_VALUE lhs, BOOL3_VALUE rhs)
	{ return ((lhs == BOOL3_False || rhs == BOOL3_False) ? BOOL3_False : (lhs == BOOL3_True && rhs == BOOL3_True) ? BOOL3_True : BOOL3_Unknown); }

//! Logical or.
inline BOOL3_VALUE operator|| (BOOL3_VALUE lhs, BOOL3_VALUE rhs)
	{ return ((lhs == BOOL3_True || rhs == BOOL3_True) ? BOOL3_True : (lhs == BOOL3_False && rhs == BOOL3_False) ? BOOL3_False : BOOL3_Unknown); }


//! Container for unknown/false/true value.
//! Size is 1 byte, values are -1, 0, 1.
class BOOL3 {
	public:

		//! Construct with specified value, default of Unknown.
		BOOL3 (
			BOOL3_VALUE value = BOOL3_Unknown		//!< Initial value
			) : m_value(static_cast<signed char>(value)) { }

		//! Construct from bool value.
		BOOL3 (
			bool value										//!< Initial value
			) : m_value(value ? 1 : 0) { }

		//! Assign value.
		BOOL3& operator= (
			BOOL3_VALUE value								//!< Value to assign
			) { m_value = static_cast<signed char>(value); return (*this); }

		//! Assign from bool value.
		BOOL3& operator= (
			bool value										//!< Value to assign.
			) { m_value = (value ? 1 : 0); return (*this); }

		//! Cast to BOOL3_VALUE.
		operator BOOL3_VALUE (
			) const { return (static_cast<BOOL3_VALUE>(m_value)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		signed char m_value;
		#endif
	};

#endif	// INC_MI32_BOOL3_H
