/**
 * \file mi32/matlas.h
 * \brief Definitions for "atlas" library functions
 *
 * \if NODOC
 * $Id: matlas.h_v 1.29 2012/05/10 13:55:15 mju Exp $
 *
 * $Log: matlas.h_v $
 * Revision 1.29  2012/05/10 13:55:15  mju
 * Add viewinfo::getSidebarVisible and corresponding bool member.
 * Remove old legendmode member.
 *
 * Revision 1.28  2011/10/26 13:01:48  mju
 * Remove redundant hdrs.
 *
 * Revision 1.27  2007/08/20 14:20:45  mju
 * Declare rvc::object.
 *
 * Revision 1.20  2005/08/17 15:31:33  mju
 * Add startup object stuff.
 *
 * Revision 1.14  2003/07/29 15:42:38  mju
 * Add SetBackgroundColor.
 *
 * Revision 1.13  2003/05/19 21:03:39  mju
 * Add SetCustomizePath.
 *
 * Revision 1.12  2003/05/14 21:19:48  mju
 * Allow access to 'Ini' handle.
 * Support interface customization.
 *
 * Revision 1.11  2003/04/29 21:18:23  mju
 * Add MATLASUSEDFILE class/list and use so can deal with RLK files.
 *
 * Revision 1.10  2003/02/18 16:37:52  mju
 * Add SetCenter method.
 *
 * Revision 1.9  2002/12/16 17:26:14  mju
 * Add multi-view open capability.
 *
 * Revision 1.8  2002/11/20 18:41:32  mju
 * Add CheckObject.
 *
 * Revision 1.7  2002/10/25 20:54:22  mju
 * Support all items settable in atlas file with methods to get.
 *
 * Revision 1.6  2002/08/22 14:55:02  mju
 * Simple MATLASERRITEM and define MATLASERRLIST.
 * Use list references in validation.
 *
 * Revision 1.3  2000/03/22 16:26:51  mju
 * Add MATLASINFO class.
 *
 * Revision 1.1  1999/06/28  15:06:30  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MATLAS_H
#define	INC_MI32_MATLAS_H

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_INIHND_H
	#include <mi32/inihnd.h>
#endif

#ifndef	INC_GRE_GEOLOCK_H
	#include <gre/geolock.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_POINT_H
	#include <mi32/point.h>
#endif

#ifndef	INC_VECTOR
#include <vector>
#define INC_VECTOR
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class OBJECT;
	}
#endif


class MATLASINFO {
	public:

		class VIEWINFO {
			public:

				struct POSITION {
					int m_xinit;
					int m_xsize;
					int m_xrange;
					int m_yinit;
					int m_ysize;
					int m_yrange;

					POSITION ();
					bool IsValid () const;
					void SerialGetString (
						char *string				//!< String to contain values, should be at least 64 bytes
						) const;
					bool SerialSetString (
						const char *string		//!< String to set from
						);
					void Set (int xinit, int xsize, int xrange, int yinit, int ysize, int yrange) {
						m_xinit = xinit; m_xsize = xsize; m_xrange = xrange;
						m_yinit = yinit; m_ysize = ysize; m_yrange = yrange;
						}
					};

				VIEWINFO (
					) : m_bSidebarVisible(true) { }

				const GRE::GEOLOCKSETTINGS& GetGeoLockSettings (
					) const { return (m_GeoLockSettings); }

				const POSITION& GetPosition (
					) const { return (m_Position); }

				bool GetSidebarVisible () const
					{ return (m_bSidebarVisible); }

				void SetGeoLockSettings (
					const GRE::GEOLOCKSETTINGS& GeoLockSettings
					) { m_GeoLockSettings = GeoLockSettings; }

				void SetPosition (
					const POSITION& Position
					) { m_Position = Position; }

			private:
   			#ifndef GENERATING_DOXYGEN_OUTPUT
				GRE::GEOLOCKSETTINGS m_GeoLockSettings;
				POSITION m_Position;
				bool m_bSidebarVisible;
				friend class MATLASINFO;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Default constructor.
		MATLASINFO (
			);

		//! Copy constructor.
		MATLASINFO (
			const MATLASINFO& rhs
			);

		//! Destructor.
		~MATLASINFO (
			);

		//! Assignment.
		MATLASINFO& operator= (
			const MATLASINFO& rhs
			);

		//! Check if object referenced by atlas actually exists.
		ERRVALUE CheckObject (
			) const;

		//! Read atlas file.
		ERRVALUE FileRead (
			const FILEPATH& filepath
			);

		//! Write atlas file.
		ERRVALUE FileWrite (
			const FILEPATH& filepath,
			bool wildcardfile = false
			);

		const COLOR& GetBackgroundColor (
			) const { return (m_BackgroundColor); }

		bool GetCenter (
			DPOINT2D& Center
			) const { Center = m_Center; return (m_HasCenter); }

		const FILEPATH& GetCustomizeFilePath (
			) const { return (m_CustomizePath); }

		const INIHND& GetIniHandle (
			) const { return (m_IniHandle); }

		const RVC::OBJITEM& GetObjectItem (
			) const { return (m_ObjItem); }

		const RVC::OBJITEM& GetObjItemStartup (
			) const { return (m_ObjItemStartup); }

		const MISTRING& GetTitle (
			) const { return (m_Title); }

		int GetViewCount (
			) const { return (m_ViewInfo.size()); }

		const VIEWINFO& GetViewInfo (
			unsigned index
			) const { return (m_ViewInfo[index]); }

		bool GetWidth (
			double& Width
			) const { Width = m_Width; return (m_HasWidth); }

		bool GetZoom1X (
			) const { return (m_Zoom1X); }

		bool IsAllowedCustomize (
			) const { return (m_AllowCustomize); }

		bool IsOpenDisabled (
			) const { return (m_IsOpenDisabled); }

		void SetAllowedCustomize (
			bool AllowCustomize
			) { m_AllowCustomize = AllowCustomize; }

		void SetBackgroundColor (
			const COLOR& BackgroundColor
			) { m_BackgroundColor = BackgroundColor; }

		void SetCenter (
			const DPOINT2D& center
			) { m_Center = center; m_HasCenter = true; }

		void SetCustomizeFilePath (
			const FILEPATH& CustomizePath
			) { m_CustomizePath = CustomizePath; }

		ERRVALUE SetObject (
			const RVC::OBJITEM& objitem
			);

		//! Set object to show during initial startup while loading layout.
		void SetObjItemStartup (
			const RVC::OBJITEM& ObjItem
			) { m_ObjItemStartup = ObjItem; }

		void SetTitle (
			const MIUNICODE *Title
			) { m_Title = Title; }

		void SetWidth (
			double Width
			) { m_Width = Width; m_HasWidth = true; }

		void SetZoom1X (
			bool Zoom1X
			) { m_Zoom1X = Zoom1X; }

	private:
	   #ifndef GENERATING_DOXYGEN_OUTPUT

		//! Explicitly initialized
		bool m_HasCenter;
		bool m_HasWidth;
		bool m_Zoom1X;
		bool m_IsOpenDisabled;
		bool m_AllowCustomize;
		double m_Width;
		//! Self initializing
		INIHND m_IniHandle;
		MISTRING m_Title;
		RVC::OBJITEM m_ObjItem;
		RVC::OBJITEM m_ObjItemStartup;
		COLOR m_BackgroundColor;
		DPOINT2D m_Center;
		std::vector<VIEWINFO> m_ViewInfo;
		FILEPATH m_CustomizePath;

		bool ReadObjItem (RVC::OBJITEM& ObjItem, const char *FileTag, const char *ObjTag);
		void WriteObjItem (RVC::OBJITEM& ObjItem, const char *FileTag, const char *ObjTag, bool wildcardfile);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

class MATLASUSEDFILE {
	public:
		MATLASUSEDFILE (const RVC::OBJECT& Object);
		MATLASUSEDFILE (const FILEPATH& filepath);
		const FILEPATH& GetPathActual () const { return (m_PathActual); }
		const FILEPATH& GetPathRLK () const { return (m_PathRLK); }
		bool HasRLK () const { return (m_HasRLK); }
	private:
	   #ifndef GENERATING_DOXYGEN_OUTPUT
		FILEPATH m_PathActual;
		FILEPATH m_PathRLK;
		bool m_HasRLK;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
typedef MILIST<MATLASUSEDFILE> MATLASUSEDFILELIST;

struct MATLASERRITEM {				//! Validation error list item
	ERRVALUE errcode;						//!< TNT error code
	RVC::OBJITEM errobjitem;			//!< Object item having error
	RVC::OBJITEM refobjitem;			//!< Object item referencing bad file/object
	};

typedef MILIST<MATLASERRITEM> MATLASERRLIST;

//-----------------------------------------------------------------------------
//!		Function prototypes
//-----------------------------------------------------------------------------

//! Validate atlas starting from specified object.
ERRVALUE MatlasValidate (
	const RVC::OBJITEM& ObjItem,			//!< Starting object
	MATLASUSEDFILELIST& UsedFileList,	//!< List of "used" files returned
	MATLASERRLIST& ErrorList				//!< List of "errors" returned
	);

#endif
