/**
 * \file numtype.h <mi32/numtype.h>
 * \brief Definitions for NUMTYPE values
 *
 * \if NODOC
 * $Id: numtype.h_v 1.4 2006/12/21 18:34:36 mju Exp $
 *
 * $Log: numtype.h_v $
 * Revision 1.4  2006/12/21 18:34:36  mju
 * Remove unused legacy defns.
 *
 * Revision 1.3  2005/08/26 15:20:08  mju
 * Add NUMTYPE enum.
 *
 * Revision 1.2  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  1999/04/28 14:38:32  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_NUMTYPE_H
#define	INC_MI32_NUMTYPE_H


//! Enumeration of numeric types which can be manipulated as a single value in
//! memory by the CPU and can be declared as built-in type in source code.
//! Enumeration values may be represented as a single byte for storage.
enum NUMTYPE {
	NUMTYPE_Invalid =					0,									//!< Invalid
	NUMTYPE_MASK_Unsigned =			0x20,								//!< Mask for unsigned integer
	NUMTYPE_MASK_Signed =			0x40,								//!< Mask for signed integer
	NUMTYPE_MASK_Integer =			0x60,								//!< Mask for any integer type
	NUMTYPE_MASK_Float =				0x80,								//!< Mask for floating-point
	NUMTYPE_UINT8 =					(NUMTYPE_MASK_Unsigned|1),	//!< Unsigned 8-bit integer
	NUMTYPE_UINT16 =					(NUMTYPE_MASK_Unsigned|2),	//!< Unsigned 16-bit integer
	NUMTYPE_UINT32 =					(NUMTYPE_MASK_Unsigned|4),	//!< Unsigned 32-bit integer
	NUMTYPE_UINT64 =					(NUMTYPE_MASK_Unsigned|8),	//!< Unsigned 64-bit integer
	NUMTYPE_SINT8 =					(NUMTYPE_MASK_Signed|1),	//!< Signed 8-bit integer
	NUMTYPE_SINT16 =					(NUMTYPE_MASK_Signed|2),	//!< Signed 16-bit integer
	NUMTYPE_SINT32 =					(NUMTYPE_MASK_Signed|4),	//!< Signed 32-bit integer
	NUMTYPE_SINT64 =					(NUMTYPE_MASK_Signed|8),	//!< Signed 64-bit integer
	NUMTYPE_FLOAT32 =					(NUMTYPE_MASK_Float|4),		//!< Floating-point 32-bit 
	NUMTYPE_FLOAT64 =					(NUMTYPE_MASK_Float|8),		//!< Floating-point 64-bit 
	NUMTYPE_MASK_Size =				0x1F								//!< Mask for size in bytes
	};

#endif	// INC_MI32_NUMTYPE_H 
