/**
 * \file imagefiltertestequal.h <spatmod/imagefiltertestequal.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_TESTEQUAL definition.
 *
 * \if NODOC
 *
 * $Id: imagefiltertestequal.h_v 1.1 2006/11/28 14:06:58 mju Exp $
 *
 * $Log: imagefiltertestequal.h_v $
 * Revision 1.1  2006/11/28 14:06:58  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERTESTEQUAL_H
#define	INC_SPATMOD_IMAGEFILTERTESTEQUAL_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to test pixel for equality.
//! Resulting image will have single sample of type UINT8 and precision 1.
class FILTER_TESTEQUAL : public FILTER {
	public:

		//! Constructor using PIXEL value to test for.
		FILTER_TESTEQUAL (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const PIXEL& value,						//!< Value to test for
			bool negate = false						//!< Negate result, output 0 if equal, 1 if unequal
			);

		//! Constructor using double value to test for.
		//! Input image must have single component.
		FILTER_TESTEQUAL (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const double value,						//!< Value to test for
			bool negate = false						//!< Negate result, output 0 if equal, 1 if unequal
			);

		//! Destructor.
		virtual ~FILTER_TESTEQUAL ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL m_PixelTest;
		bool m_Negate;

		void ProcessUINT8 (UINT8 *pTgt, const UINT8 *pSrc, UINT8 test, UINT32 NumColumns, bool firstpass) const;
		void ProcessSINT8 (UINT8 *pTgt, const INT8 *pSrc, INT8 test, UINT32 NumColumns, bool firstpass) const;
		void ProcessUINT16 (UINT8 *pTgt, const UINT16 *pSrc, UINT16 test, UINT32 NumColumns, bool firstpass) const;
		void ProcessSINT16 (UINT8 *pTgt, const INT16 *pSrc, INT16 test, UINT32 NumColumns, bool firstpass) const;
		void ProcessUINT32 (UINT8 *pTgt, const UINT32 *pSrc, UINT32 test, UINT32 NumColumns, bool firstpass) const;
		void ProcessSINT32 (UINT8 *pTgt, const INT32 *pSrc, INT32 test, UINT32 NumColumns, bool firstpass) const;
		void ProcessFloat (UINT8 *pTgt, const float *pSrc, float test, UINT32 NumColumns, bool firstpass) const;
		void ProcessDouble (UINT8 *pTgt, const double *pSrc, double test, UINT32 NumColumns, bool firstpass) const;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_TESTEQUAL (const FILTER_TESTEQUAL&);
		FILTER_TESTEQUAL& operator= (const FILTER_TESTEQUAL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERTESTEQUAL_H

