/**
 * \file spatmod/imagestagearray.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::STAGE_ARRAY definition.
 *
 * \if NODOC
 *
 * $Id: imagestagearray.h_v 1.2 2006/12/06 22:07:44 dwilliss Exp $
 *
 * $Log: imagestagearray.h_v $
 * Revision 1.2  2006/12/06 22:07:44  dwilliss
 * Added Append and Clear methods
 *
 * Revision 1.1  2006/10/11 14:09:52  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESTAGEARRAY_H
#define	INC_SPATMOD_IMAGESTAGEARRAY_H

namespace SPATMOD {
namespace IMAGE {

class STAGE;
class STAGE_AUTOPTR;

//=====================================================================================================================

//! Array of stage pointers.
//! Will delete those stages for which ownership is assigned.
class STAGE_ARRAY {
	public:

		//! Default constructor.
		STAGE_ARRAY (
			) : m_array(0), m_numitems(0) { }

		//! Construct array of specified size
		explicit STAGE_ARRAY (
			unsigned numitems
			);

		//! Destructor.
		~STAGE_ARRAY ();

		//! Array access operator.
		const STAGE_AUTOPTR& operator[] (
			unsigned idx
			) const;

		//! Array access operator.
		STAGE_AUTOPTR& operator[] (
			unsigned idx
			);

		//! Append a stage to the array
		void Append (
			STAGE* pStage,								//!< Pointer to stage
			bool TakeOwnership						//!< Take ownership
			);

		//! Append a stage to the array
		void Append (
			STAGE_AUTOPTR& StagePtr,				//!< Stage pointer
			bool TakeOwnership						//!< Take ownership from StagePtr
			);

		//! Assign from source array.
		void Assign (
			STAGE_ARRAY& source,						//!< Source array to assign from
			bool TakeOwnership						//!< Take over ownership from source
			);

		//! Clear the array.
		void Clear (
			);

		//! Get number of items in array.
		unsigned GetNumItems (
			) const { return (m_numitems); }

		//! Set individual array item with specified ownership.
		void SetItem (
			unsigned index,							//!< Item index
			STAGE* pStage,								//!< Pointer to stage
			bool TakeOwnership						//!< Take ownership
			);

		//! Set individual array item with specified ownership.
		void SetItem (
			unsigned index,							//!< Item index
			STAGE_AUTOPTR& StagePtr,				//!< Stage pointer
			bool TakeOwnership						//!< Take ownership from StagePtr
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		STAGE_AUTOPTR *m_array;
		unsigned m_numitems;

		STAGE_ARRAY (const STAGE_ARRAY&);
		STAGE_ARRAY& operator= (const STAGE_ARRAY&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESTAGEARRAY_H
